<?php
/**
 * User: VeXell
 * Date: 31.07.11
 * Time: 1:12
 */
 
class KenPropertyModelDb implements InterfaceKenPropertyData {

    private $_dbh = null;

    public function __construct( $options )
    {
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
           $dsn = "odbc:Driver={SQL Server};Server={$options['dbServer']};Database={$options['dbName']};Uid={$options['dbUser']};Pwd={$options['dbPassword']}";
        } else {
           $dsn = "dblib:host={$options['dbServer']};dbname={$options['dbName']};";
        }

        try {
            $this->_dbh = new PDO($dsn, $options['dbUser'], $options['dbPassword']);
        } catch ( Exception $e ) {
            echo 'Error';
        }

    }

    private function generateWhereText($where)
    {
        $whereText = '';
        $valuesWhere = array();

        if (!empty($where)) {
            $whereText .= ' WHERE ';
            $dataWhere = array();

            foreach ($where as $key => $value) {

                if (empty($value['data'])) {
                    continue;
                }

                if ($value['condition'] == 'IN') {

                    foreach ($value['data'] as &$tValue) {
                        $tValue = "'" . $tValue . "'";
                    }

                    $dataWhere[] = $value['field'] . ' IN ' . '(' . implode(',', $value['data']).')';

                } else if ($value['condition'] == '>') {
                    $dataWhere[] = $value['field'] . '>=' . ':' . $value['field'] . '_from';
                    $valuesWhere[':' . $value['field'] . '_from'] = $value['data'];
                } else if ($value['condition'] == '<') {
                    $dataWhere[] = $value['field'] . '<=' . ':' . $value['field'] . '_to';
                    $valuesWhere[':' . $value['field'] . '_to'] = $value['data'];
                } else if ($value['condition'] == '<>') {
                    $dataWhere[] = $value['field'] . '<>' . ':' . $value['field'];
                    $valuesWhere[':' . $value['field']] = trim($value['data']);
                } else {
                    $dataWhere[] = $value['field'] . '=' . ':' . $value['field'];
                    $valuesWhere[':' . $value['field']] = $value['data'];
                }

                /*if ($value['field'] == 'Start_Date') {
                    $dataWhere[] = $value['field'] . '>=' . ':' . $value['field'];
                    $valuesWhere[':' . $value['field']] = date('Y-m-d 00:00:00');
                }*/

            }

            $whereText .= implode(' AND ', $dataWhere );
        }


        return array(
            'whereText'     => $whereText,
            'valuesWhere'   => $valuesWhere
        );
    }

    public function getSearch($page = 1, $limit = 10, $where = array(), $orderBy=array())
    {
        $getFieldsArray = array('f.LIST_NO', 'f.bedroom_count AS no_bedrooms',
                           'f.full_bath_count AS no_full_baths', 'f.total_area AS square_feet',
                           'f.list_price AS price', 'f.photo_count', 'f.PROP_TYPE',
                           'f.address1', 'f.address2', 'f.postal_code',
                           'f.lat', 'f.lon', 'f.status', 's.Start_Date', 's.End_Date', 's.Comments', 't.gis_neighborhood_id');


        $out = array(
            'count' => 0,
            'data'  => array()
        );

        $limitFrom = $page * $limit - $limit;
        $limitTo = $page * $limit;

        $additionalJoin = '';

        ## Find by area name
        ## Get gis_neighborhood_id by name
        foreach ($where as $key => $value) {
            if ( $value['field'] == 'NEIGHBORHD' && !empty($value['data'])) {

                foreach ($value['data'] as &$tValue) {
                    $tValue = "'" . $tValue . "'";
                }

                $dataWhere = $value['field'] . ' IN ' . '(' . implode(',', $value['data']).')';

                $sql = 'SELECT gis_neighborhood_id FROM geo_neighborhood WHERE ' . $dataWhere . '';

                $stmt = $this->_dbh->prepare($sql);
                $stmt->execute();

                $result = $stmt->fetchAll();

                if ( !empty($result) ) {

                    $neighborhood = array();

                    foreach ($result as $gisId) {
                        $neighborhood[] = $gisId['gis_neighborhood_id'];
                    }

                    $where[] = array('field' => 'gis_neighborhood_id', 'condition' => 'IN', 'data' => $neighborhood);
                }
                
                unset($where[$key]);
            }
        }

        ## Find MLS by gis_neighborhood_id name
        ## Get MLS ID by name
        foreach ($where as $key => $value) {
            if ( $value['field'] == 'gis_neighborhood_id' && !empty($value['data'])) {

                $additionalJoin = ' LEFT JOIN pall_geo t ON f.LIST_NO=t.LIST_NO ';

            } else if($value['field'] == 'gis_neighborhood_id') {
                unset($where[$key]);

                $searchKey = array_search('t.gis_neighborhood_id', $getFieldsArray);

                if ($searchKey) {
                    unset($getFieldsArray[$searchKey]);
                }
            }
        }

        $whereValues = $this->generateWhereText($where);

        $whereText = $whereValues['whereText'];
        $valuesWhere = $whereValues['valuesWhere'];

        $orderByText = '';

        if (!empty($orderBy)) {
            $orderByText = $orderBy['field'] . ' ' . $orderBy['type'];
        } else {
            $orderByText = 'LIST_NO DESC';
        }

        $getFields = implode(', ', $getFieldsArray);
        ## Amazing limit. MSSQL crazy database.
        $sql = 'SELECT * FROM
                (
                   SELECT ROW_NUMBER() OVER (ORDER BY ' . $orderByText . ') as rownum, ' . $getFields . ' FROM mls_simple_search f LEFT JOIN Open_Houses s ON f.LIST_NO=s.LIST_NO ' . $additionalJoin . ' ' . $whereText . '
                ) as f
                WHERE f.rownum BETWEEN ' . $limitFrom . ' AND ' . $limitTo . '';

        //$sql = 'SELECT TOP 1 * FROM mls_simple_search_temp';

        //print_r($valuesWhere);
       // $sql = 'SELECT TOP 10 ' . $getFields . ' FROM mls_simple_search f LEFT JOIN Open_Houses s ON f.LIST_NO=s.LIST_NO LEFT JOIN pall_geo t ON f.LIST_NO=t.LIST_NO ' . $whereText . ' ORDER BY ' . $orderByText . '';
        //echo $sql;

        $stmt = $this->_dbh->prepare($sql);
        $stmt->execute($valuesWhere);

        //print_r($stmt->errorInfo());

        $out['data']  = $stmt->fetchAll();

        unset($stmt);

        $stmt = $this->_dbh->prepare('SELECT COUNT(f.LIST_NO) as propertyCount FROM mls_simple_search f LEFT JOIN Open_Houses s ON f.LIST_NO=s.LIST_NO ' . $additionalJoin . ' ' . $whereText . '');
        $stmt->execute($valuesWhere);
        
        $count = $stmt->fetch();

        $out['count'] = $count['propertyCount'];

        return $out;
    }

    public function getPropertyDetails($id)
    {
        $stmt = $this->_dbh->prepare('SELECT f.*, s.Start_Date, s.End_Date, s.Comments FROM MLS_Detail2 as f LEFT JOIN Open_Houses as s ON f.LIST_NO=s.LIST_NO WHERE f.LIST_NO = ?');
        $stmt->execute(array($id));

        return $stmt->fetch();
    }


    public function getNeighborhoodPropertyPosition($id, $where)
    {
        foreach ($where as $key => $value) {
            if ( $value['field'] == 'NEIGHBORHD' ) {
                unset($where[$key]);
            }
        }

        $whereValues = $this->generateWhereText($where);

        $whereText = $whereValues['whereText'];
        $valuesWhere = $whereValues['valuesWhere'];

        $sql = 'SELECT f.LIST_NO, f.PROP_TYPE, f.lat, f.lon,
                       f.bedroom_count,
                       f.full_bath_count, f.total_area,
                       f.list_price AS price, f.photo_count, f.PROP_TYPE,
                       f.address1 FROM mls_simple_search as f
                LEFT JOIN Open_Houses s ON f.LIST_NO=s.LIST_NO
                LEFT JOIN pall_geo as t ON f.LIST_NO = t.LIST_NO ' . $whereText;

        $stmt = $this->_dbh->prepare($sql);
        $stmt->execute($valuesWhere);

        return $stmt->fetchAll();
    }

}
